#include <math.h>
#include <GL/glut.h>

const int chekor=20;     // kolku skali da se iscrtaat na edno skalilo
float transZ=10;         // tekovna oddalechenost na pogledot
float rotateA=0;         // tekoven agol na rotacija
float rotateAspeed=0.0f; // prirast na agolot na rotacija (brzina)
int polygonMode=GL_LINE; // mod na prikaz na poligonite - so linii

/* Procedura za iscrtuvanje na edno spiralno skalilo 
 * Kje ima tochno na broj "chekor" skali i spiralata kje svrtuva dva kruga
 * visinata na skaliloto kje bide prilagodena da bide obmotana na stolb visok 1
 * no na visina 0 i 1 nema potreba od skalila, pa kje bidat "chekor" bez niv
 * radiusot na skaliloto kje bide tochno 1 (odnosno skalite kje bidat shiroki 1)
 */
void skalilo (float chekor)
{
      float stepa=4.0*M_PI / chekor; // agol koj zafakja edna skala (2 kruga)
      float stepv=1.0 / (chekor+1);  // razlika vo visina na dve skalila
      float a=0;                     // pocheten agol za ciklusot
      
      /* aktiviraj izmeni na matrica za transformacii na prostorot.
       * transformaciite treba da se sfatat kako da se kontrolira robot koj
       * znae da crta. 
       * Samiot robot e pochetokot na koordinatniot sistem.
       * Orientacijata na robotot desno mu e +x, levo -x,
       * gore mu e +y, dole mu e -y, napred e -z, nazad e +z.
       * Robot znae samo relativno da gleda vo odnos na sebe.
       * Koga se pravi nekakva transformacija vsushnost se pomestuva robotot,
       * zaedno so koordinatniot pochetok vo nekoja nova pozicija i potoa 
       * crtanjeto pochnuva od tamu.
       */
      glMatrixMode(GL_MODELVIEW);

      /* stavi ja tekovnata matrica na stack
       * shto e isto kako da mu kazheme na robotot zapamti kade si se naogjal, 
       * kon kade si bil orientiral
       */
      glPushMatrix(); 

      /* robotot odi pogore za pola chekor da pochne malku pogore prvataskala */
      glTranslatef(0,stepv,0);

      /* ciklus 2 svrtuvanja okolu kruzhnica, vo nekolku chekori:
       * vo sekoj chekor robotot kje iscrta edna skala, pa kje se svrti malku
       * vo prostorot i kje otide pogore vo prostorot za da bide tochno 
       * pozicioniran za crtanje na slednata skala
       */
      glColor3f( 1.0f,0.0f,0.0f); 
      while ( a <= 4.0*M_PI ) {

         /* nacrtaj triagolnik so bojata da se menuva po sekoj chekoj 
          * triagolnikot e vpishan vo kruzhnica so radius 1
          */
         glBegin(GL_TRIANGLES);   
            glVertex3f(0,0,0);                
            glVertex3f(1,0,0);                
            glVertex3f(cos(stepa),0,sin(stepa));
         glEnd();

         /* robotot se svrtuva tochno za agolot kolku shto e edna skala, 
          * no ovde, za razlika od cos, sin, se potrebni stepeni, a ne radijani
          * svrtuvanjeto e okolu oska shto trgnuva od (0,0,0)-(0,1,0) a toa e
          * faktichki samata y-oska
          */
         glRotatef(stepa * 180.0/M_PI, 0,1,0); 

         /* robotot odi pogore za edna skala */
         glTranslatef(0,stepv,0);

         
         a+=stepa;
      }      
      glPopMatrix(); // vrati se na poslednata zapamtena pozicija na robotot
}

/* procedura za iscrtuvanje na eden stolb so visina 1 i radius 1 (cilindar) */
void stolb (float chekor)
{
      float stepa=2.0 * M_PI/chekor;
      float a=0.0;
      
      glMatrixMode(GL_MODELVIEW);
      glPushMatrix();
      while ( !( a > 2.0*M_PI ) ) {
         /* slichno kako kaj skalite, no se koristat chetiriagolnici */
         glBegin(GL_QUADS);  
            glColor3f(1.0f,1.0f,0.0f);
            glVertex3f(1,0,0);
            glVertex3f(cos(stepa),0,sin(stepa));
            glVertex3f(cos(stepa),1,sin(stepa));
            glVertex3f(1,1,0);
         glEnd();
         glRotatef(stepa*180.0/M_PI,0,1,0);
         a+=stepa;
      }
      glPopMatrix();
}

/* soba so centar vo 0,0,0 i dadenite dimenzii, no odi od 0 pa nagore */
void soba (float dimX, float dimY, float dimZ)
{
   glMatrixMode(GL_MODELVIEW);
   glPushMatrix();

      //neophodno pomestuvanje nagore za da pochne od visina 0
      glTranslatef(0,dimY/2,0);
      //neophodno skaliranje za da ispadnat dimenziite
      glScalef(dimX/2, dimY/2, dimZ/2);
      
      // kockata e 2x2x2, so centar vo 0,0,0 (znachi od -1, do 1) 
      glBegin(GL_QUADS); 
	      //desno
         glColor3f(0.0,0.0,1);
         glVertex3f(-1,-1,-1);
         glVertex3f( 1,-1,-1);
         glVertex3f( 1, 1,-1);
         glVertex3f(-1, 1,-1);

		//levo
         glColor3f(0.9,0.9,0);
         glVertex3f(-1,-1,1);
         glVertex3f(-1, 1,1);
         glVertex3f( 1, 1,1);
         glVertex3f( 1,-1,1);

		//dole
         glColor3f(0.9,0.0,0);
         glVertex3f(-1,-1,-1);
         glVertex3f( 1,-1,-1);
         glVertex3f( 1,-1, 1);
         glVertex3f(-1,-1, 1);

         glColor3f(0.0,0.0,1);
         glVertex3f(1,-1,-1);
         glVertex3f(1, 1,-1);
         glVertex3f(1, 1, 1);
         glVertex3f(1,-1, 1);

		//gore
         glColor3f(0.0,0.0,1);
         glVertex3f(-1,1,-1);
         glVertex3f( 1,1,-1);
         glVertex3f( 1,1, 1);
         glVertex3f(-1,1, 1);

         glColor3f(0.0,0.0,1);
         glVertex3f(-1, 1,-1);
         glVertex3f(-1,-1,-1);
         glVertex3f(-1,-1, 1);
         glVertex3f(-1, 1, 1);
      glEnd();
   glPopMatrix();
}

/* 
 * Iscrtaj kombinirano stolb-skalilo
 */
void stolb_skalilo(float visina, float radiusN, float radiusV) {
   glMatrixMode(GL_MODELVIEW);
   glPushMatrix();
      glScalef(radiusV,visina,radiusV);
      stolb(chekor);
   glPopMatrix();
   glPushMatrix();
      glScalef(radiusN,visina,radiusN);
      skalilo(chekor);
   glPopMatrix();
}

void display(void)
{
   /* izbor na mod na crtanje poligoni popolneti ili prazni */
   glPolygonMode(GL_FRONT_AND_BACK,polygonMode); 

   /* brishenje na ekranot i brishenje na dlabochinskiot buffer
    * vo 3d prostor e vazhna dlabochinata na objektite vo prostorot 
    */
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 

   /* Mestenje pogled */
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   /* kamerata se naogja vo 0,0,transZ i gleda kon 0,0,0
    * nagore za kamerata e vo pravec na vektorot 0,1,0
    */
   gluLookAt(transZ*cos(rotateA),7,transZ*sin(rotateA), 0,3,0, 0,1,0); 
   
   /* 
    * crtanje scena
    */
   soba(20,10,20); // iscrtaj soba

   glTranslatef(-5,0,-5); // pomesti se na pozicija za crtanje na 1 skali
   stolb_skalilo(10.0,4.0,0.5);

   glTranslatef(10,0,0);  // pomesti se na pozicija za crtanje na 2 skali
   stolb_skalilo(10.0,4.0,0.5);

   glTranslatef(0,0,10);  // pomesti se na pozicija za crtanje na 3 skali
   stolb_skalilo(10.0,4.0,0.5);
   
   glTranslatef(-10,0,0); // pomesti se na pozicija za crtanje na 4 skali
   stolb_skalilo(10.0,4.0,0.5);

   glFlush();             // iscrtaj

   // smeni buffer za prikaz na ekran za da se eliminira trepkanje
   glutSwapBuffers();
}

void init (void) 
{ 
   glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();

   // perspektivna proekcija takva shto dimenzijata na ekranot na koj se proektira 
   // se prvite 4 koordinati kako intervali po X i po Y, 
   // potoa odi dalechinata na ekranot od okoto i na kraj maksimalnata vidliva dalechina
   glFrustum(-1, 1, -1, 1, 1, 1000); 
   // predmetite shto se preblisku i predaleku nema da bidat vidlivi!!!
           
   glEnable(GL_DEPTH_TEST); // se vklucuva sortiranje na objektite po dalechina (dlabochina)
}


void keyboard(unsigned char key, int x, int y)
{
   switch (key) {
      case 27:                 // ESC e pritisnat
         exit(0);            
         break;
      case 'W':
         transZ+=0.5f;
         glutPostRedisplay();  // obnovi go prikazot
         break;
      case 'S':
         transZ-=0.5f;
         if (transZ<0) transZ=0;
         glutPostRedisplay();  // obnovi go prikazot
         break;
      case 'w':
         transZ+=0.1f;
         glutPostRedisplay();  // obnovi go prikazot
         break;
      case 's':
         transZ-=0.1f;
         if (transZ<0) transZ=0;
         glutPostRedisplay();  // obnovi go prikazot
         break;
      case 'A':
         rotateAspeed+=1.0f;
         glutPostRedisplay();  // obnovi go prikazot
         break;
      case 'a':
         rotateAspeed+=0.01f;
         glutPostRedisplay();  // obnovi go prikazot
         break;
      case 'D':
         rotateAspeed-=1.0f;
         glutPostRedisplay();  // obnovi go prikazot
         break;
      case 'd':
         rotateAspeed-=0.01f;
         glutPostRedisplay();  // obnovi go prikazot
         break;
      case '1':
         polygonMode=GL_LINE;  // mod na prikaz za poligoni samo so rabovi
         glutPostRedisplay();  // obnovi go prikazot
         break;
      case '2':
         polygonMode=GL_FILL;  // mod na prikaz za poligoni - popolneti
         glutPostRedisplay();  // obnovi go prikazot
         break;
   }
}

void idle(void)
{
  rotateA+=rotateAspeed;
  glutPostRedisplay();     // obnovi go prikazot
}

int main(int argc, char** argv)
{
   glutInit(&argc, argv);
   glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutInitWindowSize (450, 350); 
   glutInitWindowPosition (100, 100);
   glutCreateWindow ("Soba so skalila");
   init ();
   glutDisplayFunc(display); 
   glutIdleFunc(idle); // ovaa funkcija se povikuva vo ramnomerni intervali 
                       // koga programot ne izvrshuva drugi funkcii
   glutKeyboardFunc(keyboard);  // Pri pritisok na tastaturata se povikuva keyboard
   glutMainLoop();
   return 0; 
}

